/***************************************************************************
 *   Copyright (C) 2010 by Naomasa Matsubayashi   *
 *   picosynth@quaternion.sakura.ne.jp   *
 *                                                                         *
 *   All rights reserved.                                                  *
 *                                                                         *
 * Redistribution and use in source and binary forms, with or without      *
 * modification, are permitted provided that the following conditions are  *
 * met:                                                                    *
 *                                                                         *
 *  1. Redistributions of source code must retain the above copyright      *
 *     notice, this list of conditions and the following disclaimer.       *
 *  2. Redistributions in binary form must reproduce the above copyright   *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation and/or other materials provided with the distribution.*
 *                                                                         *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS     *
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT       *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   *
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT        *
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   *
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   *
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT     *
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/

#ifndef PICOSYNTH_DRUMTRUCKRUNTIMEDATA_H
#define PICOSYNTH_DRUMTRUCKRUNTIMEDATA_H

#include <stdint.h>
#include <fixed.h>
#include <drum.h>
#include <normalize.h>
#include <truck.h>

/*
 * ドラムキット実行時データ
 * トラック実行時データのドラム版
 * ドラムキットと楽譜の現在位置情報のセット
 */
typedef struct {
  DrumKit drumkit;
  const NoteData *current_truck_pos;
} DrumTruckRuntimeData;

// ドラムキット実行時データを初期化
INLINE void dtrInit ( DrumTruckRuntimeData *_dtrd, const DrumTruck *_truck ) {
  dktInit ( &( _dtrd->drumkit ), _truck->drum );
  _dtrd->current_truck_pos = _truck->score;
}

// 楽譜の現在位置を進める
INLINE void dtrAdvance ( DrumTruckRuntimeData *_dtrd ) {
  _dtrd->current_truck_pos++;
}

// 楽譜の現在位置を得る
INLINE const NoteData *dtrGetCurrentNote ( DrumTruckRuntimeData *_dtrd ) {
  return _dtrd->current_truck_pos;
}

// ドラムキットを得る
INLINE DrumKit *dtrGetDrumKit ( DrumTruckRuntimeData *_dtrd ) {
  return & ( _dtrd->drumkit );
}

// 楽譜が存在する場合TRUEを返す
INLINE Bool dtrIsAvailable ( DrumTruckRuntimeData *_dtrd ) {
  return _dtrd->current_truck_pos != NULL;
}

// 現在の時刻でのノートオン/ノートオフイベントを処理する
INLINE void dtrControlNote ( DrumTruckRuntimeData *_dtrd, uint16_t _current_position ) {
  dktNoteOff( dtrGetDrumKit( _dtrd ), _current_position );
  for ( ; dtrGetCurrentNote ( _dtrd )->position <= _current_position; dtrAdvance ( _dtrd ) ) {
    const NoteData *note_data = dtrGetCurrentNote ( _dtrd );
    dktNoteOn ( dtrGetDrumKit( _dtrd ), note_data->noteno, _current_position );
  }
}

// 楽譜が終端に達していればTRUEを返す
INLINE Bool dtrIsEnd ( DrumTruckRuntimeData *_dtrd, uint16_t _current_position ) {
  return dtrGetCurrentNote ( _dtrd )->position <= _current_position && dtrGetCurrentNote ( _dtrd )->length == 0;
}

// 現在のサンプルを得る
INLINE Fixed8_8 dtrCalc ( DrumTruckRuntimeData *_dtrd ) {
  return dktExec ( dtrGetDrumKit( _dtrd ) );
}


#endif