/***************************************************************************
 *   Copyright (C) 2010 by Naomasa Matsubayashi   *
 *   picosynth@quaternion.sakura.ne.jp   *
 *                                                                         *
 *   All rights reserved.                                                  *
 *                                                                         *
 * Redistribution and use in source and binary forms, with or without      *
 * modification, are permitted provided that the following conditions are  *
 * met:                                                                    *
 *                                                                         *
 *  1. Redistributions of source code must retain the above copyright      *
 *     notice, this list of conditions and the following disclaimer.       *
 *  2. Redistributions in binary form must reproduce the above copyright   *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation and/or other materials provided with the distribution.*
 *                                                                         *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS     *
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT       *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   *
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT        *
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   *
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   *
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT     *
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/

#ifndef PICOSYNTH_FIXED_H
#define PICOSYNTH_FIXED_H

#include <stdint.h>

typedef int16_t Fixed16;
typedef int16_t Fixed8_8;
typedef int8_t Fixed8;

/*
 * 固定小数点数
 * Picosynthでは浮動小数点演算器のないプロセッサを想定し、固定小数点数で計算を行う
 */

INLINE Fixed8 utlMulFixed8( Fixed8 _left, Fixed8 _right ) {
  return ( (int16_t)_left * (int16_t)_right ) >> 7;
}

INLINE Fixed8_8 utlMulFixed8_8( Fixed8_8 _left, Fixed8_8 _right ) {
  return ( (int32_t)_left * (int32_t)_right ) >> 8;
}

INLINE Fixed16 utlMulFixed16( Fixed16 _left, Fixed16 _right ) {
  return ( (int32_t)_left * (int32_t)_right ) >> 15;
}

#endif