/***************************************************************************
 *   Copyright (C) 2010 by Naomasa Matsubayashi   *
 *   picosynth@quaternion.sakura.ne.jp   *
 *                                                                         *
 *   All rights reserved.                                                  *
 *                                                                         *
 * Redistribution and use in source and binary forms, with or without      *
 * modification, are permitted provided that the following conditions are  *
 * met:                                                                    *
 *                                                                         *
 *  1. Redistributions of source code must retain the above copyright      *
 *     notice, this list of conditions and the following disclaimer.       *
 *  2. Redistributions in binary form must reproduce the above copyright   *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation and/or other materials provided with the distribution.*
 *                                                                         *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS     *
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT       *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   *
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT        *
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   *
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   *
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT     *
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/

#ifndef PICOSYNTH_LINUX_H
#define PICOSYNTH_LINUX_H

#define FREQUENCY 4000ul
#define PLAYBACK_FREQUENCY 48000ul
#define INLINE extern inline

#include <types.h>
#include <fixed.h>

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>

// ALSAから出力するための関数

// デバッグ出力
INLINE void dbgPrint ( const char *_string ) {
  puts ( _string );
}

#include <alsa/asoundlib.h>

typedef struct {
  snd_pcm_t *pcm;
} PCM;

// PCMを開く
INLINE Bool pcmOpen ( PCM *_pcm ) {
  const static char device[] = "hw:0";
  const static snd_pcm_format_t format = SND_PCM_FORMAT_S16;
  const static snd_pcm_access_t access = SND_PCM_ACCESS_RW_INTERLEAVED;
  const static unsigned int sampling_rate = PLAYBACK_FREQUENCY;
  const static unsigned int channels = 1;
  const static unsigned int soft_resample = 1;
  const static unsigned int latency = 40000;
  if ( snd_pcm_open ( & ( _pcm->pcm ), device, SND_PCM_STREAM_PLAYBACK, 0 ) ) {
    dbgPrint ( "openPCM: Unable to open." );
    return FALSE;
  }
  if (
    snd_pcm_set_params (
      _pcm->pcm, format, access,
      channels, sampling_rate, soft_resample, latency
    )
  ) {
    dbgPrint ( "openPCM: Unable to set format." );
    snd_pcm_close ( _pcm->pcm );
    return FALSE;
  }
  return TRUE;
}

// PCMを書く
INLINE Bool pcmWrite ( PCM *_pcm, Fixed8 _buffer ) {
  int16_t temp = (int16_t)_buffer << 8;
  int down_rate;
  for( down_rate = 0; down_rate != PLAYBACK_FREQUENCY / FREQUENCY; down_rate++ ) {
    int write_result = snd_pcm_writei ( _pcm->pcm, &temp, 1 );
    if ( write_result < 0 ) {
      dbgPrint ( snd_strerror ( write_result ) );
      if ( snd_pcm_recover ( _pcm->pcm, write_result, 0 ) < 0 ) {
        dbgPrint ( "writePCM: Unable to recover this stream." );
        return FALSE;
      }
    }
  }
  return TRUE;
}

// PCMを閉じる
INLINE void pcmClose ( PCM *_pcm ) {
  snd_pcm_close ( _pcm->pcm );
}

#endif
