/***************************************************************************
 *   Copyright (C) 2010 by Naomasa Matsubayashi   *
 *   picosynth@quaternion.sakura.ne.jp   *
 *                                                                         *
 *   All rights reserved.                                                  *
 *                                                                         *
 * Redistribution and use in source and binary forms, with or without      *
 * modification, are permitted provided that the following conditions are  *
 * met:                                                                    *
 *                                                                         *
 *  1. Redistributions of source code must retain the above copyright      *
 *     notice, this list of conditions and the following disclaimer.       *
 *  2. Redistributions in binary form must reproduce the above copyright   *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation and/or other materials provided with the distribution.*
 *                                                                         *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS     *
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT       *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   *
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT        *
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   *
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   *
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT     *
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/

#ifndef PICOSYNTH_LINUXWAVE_H
#define PICOSYNTH_LINUXWAVE_H

#define FREQUENCY 4000ul
#define PLAYBACK_FREQUENCY 48000ul
#define INLINE extern inline

#include <types.h>
#include <fixed.h>

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>

// RIFF WAVEに出力するための関数

// デバッグ出力
INLINE void dbgPrint ( const char *_string ) {
  puts ( _string );
}

#include <alsa/asoundlib.h>

typedef struct {
  FILE *file;
  uint32_t sample_count;
} PCM;

// PCMを開く
INLINE Bool pcmOpen ( PCM *_pcm ) {
  _pcm->file = fopen( "dump.wav", "w" );
  if( !_pcm->file )
    return FALSE;
  const static uint8_t header[] = {
    'R', 'I', 'F', 'F',
    0x00, 0x00, 0x10, 0x00,
    'W', 'A', 'V','E',
    'f', 'm', 't', ' ',
    0x10, 0x00, 0x00, 0x00,
    0x01, 0x00,
    0x01, 0x00,
    (uint8_t)PLAYBACK_FREQUENCY, (uint8_t)( PLAYBACK_FREQUENCY >> 8 ), (uint8_t)( PLAYBACK_FREQUENCY >> 16 ), (uint8_t)( PLAYBACK_FREQUENCY >> 24 ),
     (uint8_t)( PLAYBACK_FREQUENCY << 1 ), (uint8_t)( PLAYBACK_FREQUENCY >> 7 ), (uint8_t)( PLAYBACK_FREQUENCY >> 15 ), (uint8_t)( PLAYBACK_FREQUENCY >> 23 ),
    0x02, 0x00,
    0x10, 0x00,
    'd', 'a', 't', 'a',
    0x00, 0x00, 0x10, 0x00,
  };
  fwrite( header, 1, 48, _pcm->file );
  _pcm->sample_count = 0;
  return TRUE;
}

// PCMを書く
INLINE Bool pcmWrite ( PCM *_pcm, Fixed8 _buffer ) {
  Fixed16 temp = (Fixed16)_buffer << 8;
  int down_rate;
  for( down_rate = 0; down_rate != PLAYBACK_FREQUENCY / FREQUENCY; down_rate++ ) {
    fwrite( &temp, 2, 1, _pcm->file );
  }
  _pcm->sample_count+=2 * PLAYBACK_FREQUENCY / FREQUENCY;
  return TRUE;
}

// PCMを閉じる
INLINE void pcmClose ( PCM *_pcm ) {
  uint32_t file_size = _pcm->sample_count + 36;

  fseek( _pcm->file, 4, SEEK_SET );
  fputc( file_size & 0xFF, _pcm->file );
  fputc( ( file_size >> 8 ) & 0xFF, _pcm->file );
  fputc( ( file_size >> 16 ) & 0xFF, _pcm->file );
  fputc( ( file_size >> 24 ) & 0xFF, _pcm->file );
  fseek( _pcm->file, 40, SEEK_SET );
  fputc( _pcm->sample_count & 0xFF, _pcm->file );
  fputc( ( _pcm->sample_count >> 8 ) & 0xFF, _pcm->file );
  fputc( ( _pcm->sample_count >> 16 ) & 0xFF, _pcm->file );
  fputc( ( _pcm->sample_count >> 24 ) & 0xFF, _pcm->file );
  fclose( _pcm->file );
}

#endif
