/***************************************************************************
 *   Copyright (C) 2010 by Naomasa Matsubayashi   *
 *   picosynth@quaternion.sakura.ne.jp   *
 *                                                                         *
 *   All rights reserved.                                                  *
 *                                                                         *
 * Redistribution and use in source and binary forms, with or without      *
 * modification, are permitted provided that the following conditions are  *
 * met:                                                                    *
 *                                                                         *
 *  1. Redistributions of source code must retain the above copyright      *
 *     notice, this list of conditions and the following disclaimer.       *
 *  2. Redistributions in binary form must reproduce the above copyright   *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation and/or other materials provided with the distribution.*
 *                                                                         *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS     *
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT       *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   *
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT        *
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   *
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   *
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT     *
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/

#ifndef PICOSYNTH_MODULE_H
#define PICOSYNTH_MODULE_H

#include <types.h>
#include <fixed.h>
#include <envelope.h>
#include <wave.h>
/*
 * モジュール
 * 音色の定義
 * 音色は2つの波形テーブルをミキシングし、エンベロープを掛けて決定される
 */
typedef struct {
  const char name[ 9 ];
  const Wave wave[ 2 ];
  Envelope envelope;
} Module;

// 波形テーブルを得る
INLINE const Wave *mdrGetWave( const Module *_module, unsigned char _select ) {
  return _module->wave + _select;
}

// エンベロープを得る
INLINE const Envelope *mdrGetEnvelope( const Module *_module ) {
  return &( _module->envelope );
}

// 現在のサンプルを得る
INLINE Fixed8 mdrExec( const Module *_module, Clock *_clock, Note *_note ) {
  Fixed8 sum = 0;
  uint8_t wav_index;
  uint8_t shift = 0;
  sum += wavExec( mdrGetWave( _module, 0 ), _clock, _note, 0 );
  if( wavIsAvailable( mdrGetWave( _module, 1 ) ) ) {
    sum >>= 1;
    sum += wavExec( mdrGetWave( _module, 1 ), _clock, _note, 1 ) >> 1;
  }
  sum = utlMulFixed8( sum, evlExec( mdrGetEnvelope( _module ), _clock, _note ) );
  return sum;
}

#endif