/***************************************************************************
 *   Copyright (C) 2010 by Naomasa Matsubayashi   *
 *   picosynth@quaternion.sakura.ne.jp   *
 *                                                                         *
 *   All rights reserved.                                                  *
 *                                                                         *
 * Redistribution and use in source and binary forms, with or without      *
 * modification, are permitted provided that the following conditions are  *
 * met:                                                                    *
 *                                                                         *
 *  1. Redistributions of source code must retain the above copyright      *
 *     notice, this list of conditions and the following disclaimer.       *
 *  2. Redistributions in binary form must reproduce the above copyright   *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation and/or other materials provided with the distribution.*
 *                                                                         *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS     *
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT       *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   *
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT        *
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   *
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   *
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT     *
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/

#ifndef PICOSYNTH_MONOPHONY_H
#define PICOSYNTH_MONOPHONY_H

#include <stdint.h>
#include <clock.h>
#include <note.h>

typedef uint16_t ExpressNo;

/*
 * 単音
 * 単音の発音に必要なリソースをまとめたもの
 */
typedef struct {
  Clock clock;
  Note note;
  uint16_t note_off_time;
} Monophony;

// モノフォニーを初期化する
INLINE void mpyInit( Monophony *_monophony ) {
  clkInit( &(_monophony->clock ) );
  nteInit( &( _monophony->note ) );
  _monophony->note_off_time = 0;
}

// クロックを得る
INLINE Clock *mpyGetClock( Monophony *_monophony ) {
  return &( _monophony->clock );
}

// 音階を得る
INLINE Note *mpyGetNote( Monophony *_monophony ) {
  return &( _monophony->note );
}

// ノートオフすべき時刻を設定する
INLINE void mpySetNoteOffTime( Monophony *_monophony, uint16_t _note_off_time ) {
  _monophony->note_off_time = _note_off_time;
}

// ノートオフすべき時刻を得る
INLINE uint16_t mpyGetNoteOffTime( Monophony *_monophony ) {
  return _monophony->note_off_time;
}

#endif
