/***************************************************************************
 *   Copyright (C) 2010 by Naomasa Matsubayashi   *
 *   picosynth@quaternion.sakura.ne.jp   *
 *                                                                         *
 *   All rights reserved.                                                  *
 *                                                                         *
 * Redistribution and use in source and binary forms, with or without      *
 * modification, are permitted provided that the following conditions are  *
 * met:                                                                    *
 *                                                                         *
 *  1. Redistributions of source code must retain the above copyright      *
 *     notice, this list of conditions and the following disclaimer.       *
 *  2. Redistributions in binary form must reproduce the above copyright   *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation and/or other materials provided with the distribution.*
 *                                                                         *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS     *
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT       *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   *
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT        *
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   *
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   *
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT     *
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/

#ifndef PICOSYNTH_NORMALIZE_H
#define PICOSYNTH_NORMALIZE_H

#include <stdint.h>
#include <fixed.h>

INLINE uint16_t utlAbsShort( int16_t _value ) {
  if( _value < 0 )
    return -_value;
  else
    return _value;
}

/*
 * 正規化
 * ダイナミックレンジ圧縮を行う
 */
typedef struct {
  uint16_t highest;
  uint8_t keep;
} Normalize;

// 正規化を初期化する
INLINE void nmlInit( Normalize *_normalize ) {
  _normalize->highest = 0;
  _normalize->keep = 0;
}

// 入力値をダイナミックレンジ圧縮して出力する
INLINE Fixed8 nmlExec( Normalize *_normalize, Fixed8_8 _value ) {
  if( _normalize->keep )
    _normalize->keep--;
  if( _normalize->highest )
    _normalize->highest--;
  uint16_t abs_value = utlAbsShort( _value );
  uint16_t level;
  if( abs_value > _normalize->highest ) {
    _normalize->highest = (uint32_t)abs_value;
    _normalize->keep = 255;
    level = abs_value >> 7;
  }
  else
    level = _normalize->highest >> 7;
  _value >>= 1;
  if( level )
    _value /= level;
  return _value;
}

#endif
