/***************************************************************************
 *   Copyright (C) 2010 by Naomasa Matsubayashi   *
 *   picosynth@quaternion.sakura.ne.jp   *
 *                                                                         *
 *   All rights reserved.                                                  *
 *                                                                         *
 * Redistribution and use in source and binary forms, with or without      *
 * modification, are permitted provided that the following conditions are  *
 * met:                                                                    *
 *                                                                         *
 *  1. Redistributions of source code must retain the above copyright      *
 *     notice, this list of conditions and the following disclaimer.       *
 *  2. Redistributions in binary form must reproduce the above copyright   *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation and/or other materials provided with the distribution.*
 *                                                                         *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS     *
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT       *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   *
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT        *
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   *
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   *
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT     *
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/

#ifndef PICOSYNTH_PLAYER_H
#define PICOSYNTH_PLAYER_H

#include <stdint.h>
#include <fixed.h>
#include <truckruntimedata.h>
#include <normalize.h>
#include <truck.h>
#include <score.h>
#include <playerruntimedata.h>


// プレイヤーを初期化する
INLINE void plyInit( Player *_player, const Score *_score ) {
  _player->score = _score;
  _player->sample_per_read = FREQUENCY * 60ul / ( (uint32_t)_player->score->tempo * 8ul );
}

// プレイヤーを実行する
INLINE void plyExec( Player *_player ) {
  PlayerRuntimeData prd;
  for( prdInit( &prd, _player ); !prdIsEnd( &prd ); prdAdvance( &prd ) ) {
    prdControlNote( &prd );
    uint16_t sample_count;
    for( sample_count = 0; sample_count != _player->sample_per_read; sample_count++ ) {
      prdCalc( &prd );
    }
  }
  prdDestroy( &prd );
}

#endif
