/***************************************************************************
 *   Copyright (C) 2010 by Naomasa Matsubayashi   *
 *   picosynth@quaternion.sakura.ne.jp   *
 *                                                                         *
 *   All rights reserved.                                                  *
 *                                                                         *
 * Redistribution and use in source and binary forms, with or without      *
 * modification, are permitted provided that the following conditions are  *
 * met:                                                                    *
 *                                                                         *
 *  1. Redistributions of source code must retain the above copyright      *
 *     notice, this list of conditions and the following disclaimer.       *
 *  2. Redistributions in binary form must reproduce the above copyright   *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation and/or other materials provided with the distribution.*
 *                                                                         *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS     *
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT       *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   *
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT        *
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   *
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   *
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT     *
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/

#ifndef PICOSYNTH_TRUCK_H
#define PICOSYNTH_TRUCK_H

#include <types.h>
#include <module.h>
#include <drum.h>

/*
 * 音符
 */
typedef struct {
  unsigned short int position;
  NoteNo noteno;
  unsigned char length;
} NoteData;

#define NOTE( _position, _length, _noteno ) \
  { .position = ( _position * 8.0 ), .length = ( _length * 8.0 ), .noteno = ( _noteno ) }

#define NOTEL( _position, _length, _noteno ) \
  { .position = ( _position * 8.0 ), .length = ( _length * 8.0 ), .noteno = ( _noteno ) - 12 }

#define NOTE_LENGTH( _num ) \
  ( 4.0f / _num )

#define TONE( _octerve, _tone ) \
  ( _octerve * 12 + _tone )
  
#define N2( _position, _length, _noteno ) \
  { .position = ( _position * 32.0 ), .length = ( _length * 32.0 ), .noteno = ( _noteno ) }

#define NL( _num ) \
  ( 1.0f / _num )

/*
 * トラック
 * 楽譜とその楽譜を演奏するモジュール
 */
typedef struct {
  const Module *module;
  const NoteData *score;
} Truck;

/*
 * ドラムトラック
 * ドラム楽譜とその楽器を演奏するドラム
 */
typedef struct {
  const Drum **drum;
  const NoteData *score;
} DrumTruck;

#endif
