/***************************************************************************
 *   Copyright (C) 2010 by Naomasa Matsubayashi   *
 *   picosynth@quaternion.sakura.ne.jp   *
 *                                                                         *
 *   All rights reserved.                                                  *
 *                                                                         *
 * Redistribution and use in source and binary forms, with or without      *
 * modification, are permitted provided that the following conditions are  *
 * met:                                                                    *
 *                                                                         *
 *  1. Redistributions of source code must retain the above copyright      *
 *     notice, this list of conditions and the following disclaimer.       *
 *  2. Redistributions in binary form must reproduce the above copyright   *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation and/or other materials provided with the distribution.*
 *                                                                         *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS     *
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT       *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   *
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT        *
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   *
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   *
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT     *
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/

#ifndef PICOSYNTH_TRUCKRUNTIMEDATA_H
#define PICOSYNTH_TRUCKRUNTIMEDATA_H

#include <stdint.h>
#include <fixed.h>
#include <instrument.h>
#include <normalize.h>
#include <truck.h>

/*
 * トラック実行時データ
 * トラック再生時に必要になる情報をまとめたもの
 */
typedef struct {
  Instrument instrument;
  const NoteData *current_truck_pos;
} TruckRuntimeData;

// トラック実行時データを初期化する
INLINE void trdInit ( TruckRuntimeData *_trd, const Truck *_truck ) {
  istInit ( & ( _trd->instrument ), _truck->module );
  _trd->current_truck_pos = _truck->score;
}

// 現在時刻を進める
INLINE void trdAdvance ( TruckRuntimeData *_trd ) {
  _trd->current_truck_pos++;
}

// 現在の音符を得る
INLINE const NoteData *trdGetCurrentNote ( TruckRuntimeData *_trd ) {
  return _trd->current_truck_pos;
}

// 楽器を得る
INLINE Instrument *trdGetInstrument ( TruckRuntimeData *_trd ) {
  return & ( _trd->instrument );
}

// 楽譜が存在するかどうか調べる
INLINE Bool trdIsAvailable ( TruckRuntimeData *_trd ) {
  return _trd->current_truck_pos != NULL;
}

// 現在の時刻でのノートオン/ノートオフイベントを処理する
INLINE void trdControlNote ( TruckRuntimeData *_trd, uint16_t _current_position ) {
  istNoteOff( trdGetInstrument( _trd ), _current_position );
  for ( ; trdGetCurrentNote ( _trd )->position <= _current_position; trdAdvance ( _trd ) ) {
    const NoteData *note_data = trdGetCurrentNote ( _trd );
    istNoteOn ( trdGetInstrument ( _trd ), note_data->noteno, _current_position + note_data->length );
  }
}

// 楽譜が終端に達していたらTRUEを返す
INLINE Bool trdIsEnd ( TruckRuntimeData *_trd, uint16_t _current_position ) {
  return trdGetCurrentNote ( _trd )->position <= _current_position && trdGetCurrentNote ( _trd )->length == 0;
}

// 現在の時刻での値を得る
INLINE Fixed8_8 trdCalc ( TruckRuntimeData *_trd ) {
  return istExec ( trdGetInstrument ( _trd ) );
}


#endif
